// Main.cpp
//
// This source file is a supplement to the article "Digitalni zvuk". The article
// was published in the #34 issue of the "PC" computer monthly, published in
// Belgrade, Yugoslavia by "PC Press"
//
// You can freely use, modify and redistribute this file, provided that this
// header is kept unchanged
//
//                                                                 Ivica Nikolic
//                                                      ivica@galeb.etf.bg.ac.yu
//
////////////////////////////////////////////////////////////////////////////////

#include "StdInclude.h"

#include "RecorderThread.h"
#include "PlayerThread.h"
#include "ProcessorThread.h"

//#define F_SAMP      8000
//#define F_SAMP      11025
//#define F_SAMP      22050
#define F_SAMP      44100

#define BITS_PER_SAMPLE 16

#define NO_OF_CHANNELS 2

#define BYTES_PER_SAMPLE ((BITS_PER_SAMPLE-1) / 8 + 1)

#define BUFFER_SIZE	(NO_OF_CHANNELS * BYTES_PER_SAMPLE * F_SAMP / 5)
#define NO_OF_IO_BUFFERS 3
#define NO_OF_RING_BUFFERS 3

void main( void )
{
	WAVEFORMATEX wfmt;
	wfmt.wFormatTag = WAVE_FORMAT_PCM;
	wfmt.nChannels = NO_OF_CHANNELS;
	wfmt.nSamplesPerSec = F_SAMP;
	wfmt.nAvgBytesPerSec = NO_OF_CHANNELS * BYTES_PER_SAMPLE * F_SAMP;
	wfmt.nBlockAlign = NO_OF_CHANNELS * BYTES_PER_SAMPLE;
	wfmt.wBitsPerSample = BITS_PER_SAMPLE;
	wfmt.cbSize = 0;

	CRecorderThread* pthrdRecorder = new CRecorderThread(
		&wfmt,
		BUFFER_SIZE,
		NO_OF_IO_BUFFERS,
		NO_OF_RING_BUFFERS
	);

	CPlayerThread* pthrdPlayer = new CPlayerThread(
		&wfmt,
		BUFFER_SIZE,
		NO_OF_IO_BUFFERS,
		NO_OF_RING_BUFFERS
	);

	CProcessorThread* pthrdProcessor = new CProcessorThread(
		&wfmt,
		BUFFER_SIZE,
		pthrdRecorder,
		pthrdPlayer
	);

	if ( !GE_INST.HasOccured() )
	{
		pthrdRecorder->ResumeThread();
		pthrdPlayer->ResumeThread();
		pthrdProcessor->ResumeThread();

		cprintf( "Filter started: sampling rate %dHz, %d bits per sample, %d channels\n",
			F_SAMP, BITS_PER_SAMPLE, NO_OF_CHANNELS );
		cprintf( "Press space to finish\n" );
	}

	BOOL bShouldContinue = !GE_INST.HasOccured();
	while ( bShouldContinue )
	{
		char c = (char)_getch();
		if ( c==' ' )
			bShouldContinue = FALSE;
	}

	if ( !GE_INST.HasOccured() )
	{
		pthrdProcessor->SetFinishEvent();
		pthrdRecorder->SetFinishEvent();
		pthrdPlayer->SetFinishEvent();

		HANDLE hThreadHandles[3] = {
			pthrdRecorder->GetThreadHandle(),
			pthrdPlayer->GetThreadHandle(),
			pthrdProcessor->GetThreadHandle()
		};

		WaitForMultipleObjects( 3, hThreadHandles, TRUE, INFINITE );
	}

	delete pthrdRecorder;
	delete pthrdPlayer;
	delete pthrdProcessor;
}
