// PlayerThread.cpp: implementation of the CPlayerThread class.
//
// This source file is a supplement to the article "Digitalni zvuk". The article
// was published in the #34 issue of the "PC" computer monthly, published in
// Belgrade, Yugoslavia by "PC Press"
//
// You can freely use, modify and redistribute this file, provided that this
// header is kept unchanged
//
//                                                                 Ivica Nikolic
//                                                      ivica@galeb.etf.bg.ac.yu
//
////////////////////////////////////////////////////////////////////////////////

#include "StdInclude.h"
#include "PlayerThread.h"

CPlayerThread::CPlayerThread(
	WAVEFORMATEX* pwfmt,
	UINT nBufferSize,
	UINT nIOBuffers,
	UINT nRingBuffers
) :
	CWaveIOThread( pwfmt, nBufferSize, nIOBuffers, nRingBuffers )
{
	m_hwo = NULL;

	if ( GE_INST.HasOccured() )
		return;

	MMRESULT mmr = waveOutOpen(
		&m_hwo,
		WAVE_MAPPER,
		&m_wfmt,
		(DWORD)m_hEventIOCompletion,
		NULL,
		CALLBACK_EVENT
	);

	if ( mmr!=MMSYSERR_NOERROR )
	{
		cprintf( "Can not open wave out device\n" );
		GE_INST.Raise();
	}
}

CPlayerThread::~CPlayerThread()
{
	if ( m_hwo!=NULL )
	{
		waveOutReset( m_hwo );
		waveOutClose( m_hwo );
	}
}

void CPlayerThread::Run()
{
	if ( GE_INST.HasOccured() )
		return;

	for ( UINT nBuffer=0; nBuffer<m_nIOBuffers; nBuffer++ )
		SubmitOutputBuffer( nBuffer, FALSE );

	ResetEvent( m_hEventIOCompletion );

	while ( !GE_INST.HasOccured() && !IsFinishEventSet() )
	{
		BOOL bShouldResetEvent;
		UINT nBuffer = WaitForEvent( bShouldResetEvent );
		if ( nBuffer==-1 )
			break;

		waveOutUnprepareHeader( m_hwo, &m_pwhdrIOBuffers[nBuffer],
			sizeof(m_pwhdrIOBuffers[nBuffer]) );

		m_buff.GetBlock( (BYTE*)m_pwhdrIOBuffers[nBuffer].lpData );

		SubmitOutputBuffer( nBuffer, bShouldResetEvent );
	}

	waveOutReset( m_hwo );

	if ( m_buff.IsFull() )
		m_buff.GetBlock( (BYTE*)m_pwhdrIOBuffers[0].lpData );
}

void CPlayerThread::SubmitOutputBuffer( UINT nBuffer, BOOL bShouldResetEvent )
{
	WAVEHDR* pwhdr = &m_pwhdrIOBuffers[ nBuffer ];

	pwhdr->dwBytesRecorded = 0;
	pwhdr->dwUser = 0;
	pwhdr->dwFlags = 0;
	pwhdr->dwLoops = 0;
	pwhdr->lpNext = NULL;
	pwhdr->reserved = 0;

	waveOutPrepareHeader( m_hwo, pwhdr, sizeof(*pwhdr) );

	if ( bShouldResetEvent )
		ResetEvent( m_hEventIOCompletion );

	waveOutWrite( m_hwo, pwhdr, sizeof(*pwhdr) );
}
