// RecorderThread.cpp: implementation of the CRecorderThread class.
//
// This source file is a supplement to the article "Digitalni zvuk". The article
// was published in the #34 issue of the "PC" computer monthly, published in
// Belgrade, Yugoslavia by "PC Press"
//
// You can freely use, modify and redistribute this file, provided that this
// header is kept unchanged
//
//                                                                 Ivica Nikolic
//                                                      ivica@galeb.etf.bg.ac.yu
//
////////////////////////////////////////////////////////////////////////////////

#include "StdInclude.h"
#include "RecorderThread.h"

CRecorderThread::CRecorderThread(
	WAVEFORMATEX* pwfmt,
	UINT nBufferSize,
	UINT nIOBuffers,
	UINT nRingBuffers
) :
	CWaveIOThread( pwfmt, nBufferSize, nIOBuffers, nRingBuffers )
{
	m_hwi = NULL;

	if ( GE_INST.HasOccured() )
		return;

	MMRESULT mmr = waveInOpen(
		&m_hwi,
		WAVE_MAPPER,
		&m_wfmt,
		(DWORD)m_hEventIOCompletion,
		NULL,
		CALLBACK_EVENT
	);

	if ( mmr!=MMSYSERR_NOERROR )
	{
		cprintf( "Can not open wave in device\n" );
		GE_INST.Raise();
	}
}

CRecorderThread::~CRecorderThread()
{
	if ( m_hwi!=NULL )
	{
		waveInReset( m_hwi );
		waveInClose( m_hwi );
	}
}

void CRecorderThread::Run()
{
	if ( GE_INST.HasOccured() )
		return;

	for ( UINT nBuffer=0; nBuffer<m_nIOBuffers; nBuffer++ )
		SubmitInputBuffer( nBuffer, FALSE );

	waveInStart( m_hwi );

	ResetEvent( m_hEventIOCompletion );

	while ( !GE_INST.HasOccured() && !IsFinishEventSet() )
	{
		BOOL bShouldResetEvent;
		UINT nBuffer = WaitForEvent( bShouldResetEvent );
		if ( nBuffer==-1 )
			break;

		waveInUnprepareHeader( m_hwi, &m_pwhdrIOBuffers[nBuffer],
			sizeof(m_pwhdrIOBuffers[nBuffer]) );

		m_buff.PutBlock( (BYTE*)m_pwhdrIOBuffers[nBuffer].lpData );

		SubmitInputBuffer( nBuffer, bShouldResetEvent );
	}

	waveInReset( m_hwi );

	if ( m_buff.IsEmpty() )
		m_buff.PutBlock( (BYTE*)m_pwhdrIOBuffers[0].lpData );
}

void CRecorderThread::SubmitInputBuffer( UINT nBuffer, BOOL bShouldResetEvent )
{
	WAVEHDR* pwhdr = &m_pwhdrIOBuffers[ nBuffer ];

	pwhdr->dwBytesRecorded = 0;
	pwhdr->dwUser = 0;
	pwhdr->dwFlags = 0;
	pwhdr->dwLoops = 0;
	pwhdr->lpNext = NULL;
	pwhdr->reserved = 0;

	waveInPrepareHeader( m_hwi, pwhdr, sizeof(*pwhdr) );

	if ( bShouldResetEvent )
		ResetEvent( m_hEventIOCompletion );

	waveInAddBuffer( m_hwi, pwhdr, sizeof(*pwhdr) );
}
